#ifndef __CARET_DATA_FILE_HELPER_H__
#define __CARET_DATA_FILE_HELPER_H__

/*LICENSE_START*/
/*
 *  Copyright (C) 2014  Washington University School of Medicine
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
/*LICENSE_END*/

#include "AString.h"
#include "DataFileTypeEnum.h"

namespace caret {

    class CaretDataFile;
    
    class CaretDataFileHelper {
        
    public:
        static CaretDataFile* readAnyCaretDataFile(const AString& filename, const bool& preferOnDisk = false);
        
        static AString createBadAllocExceptionMessage(const AString& filename);
        
        static DataFileTypeEnum::Enum getDataFileTypeFromFileContent(const AString& filename);
        
        static CaretDataFile* createCaretDataFileForFileType(const DataFileTypeEnum::Enum dataFileType);
        
    private:
        CaretDataFileHelper();
        
        virtual ~CaretDataFileHelper();
        
        CaretDataFileHelper(const CaretDataFileHelper&);

        CaretDataFileHelper& operator=(const CaretDataFileHelper&);
        
        static void readCaretDataFile(CaretDataFile* caretDataFile,
                                      const AString& filename,
                                      const bool& preferOnDisk);
        
    public:

        // ADD_NEW_METHODS_HERE

    private:
        // ADD_NEW_MEMBERS_HERE

    };
    
#ifdef __CARET_DATA_FILE_HELPER_DECLARE__
    // <PLACE DECLARATIONS OF STATIC MEMBERS HERE>
#endif // __CARET_DATA_FILE_HELPER_DECLARE__

} // namespace
#endif  //__CARET_DATA_FILE_HELPER_H__
